clc; clear;

%% import text file & save into an int array
fileID = fopen('testRoute.txt');
inputTxt = textscan(fileID,'%c'); % read each line character by character, spaces are not read
fclose(fileID);
%celldisp(inputTxt);

inputTxtChar = inputTxt{1};
len = length(inputTxtChar);
distance = ''; % initialize a string that will hold the distance information
distanceVal = ''; % values that will be added to distance array

for i=1:len
    num = str2num(inputTxtChar(i)); % changes a string to a number
    if(isnumeric(num) && isempty(num)==0 && isreal(num)) % 'i' is converted to a complex number, therefore use isreal()
        %disp('Is a number!'); disp(i);
        if(i~=1 && strcmp(inputTxtChar(i-1),'!')==0) % if previous value was not equal to !
            distanceVal = strcat(distanceVal,inputTxtChar(i)); % is a number
        end
        
    elseif(strcmp(inputTxtChar(i),'.'))
        %disp('Is a period!'); disp(i);
        distanceVal = strcat(distanceVal,inputTxtChar(i));
        
    elseif(strcmp(inputTxtChar(i),'('))
        %disp('Is an open bracket!'); disp(i);
        distanceVal = strcat(distanceVal,inputTxtChar(i));
        
    elseif(strcmp(inputTxtChar(i),')'))
        %disp('Is a closing bracket!'); disp(i);
        distanceVal = strcat(distanceVal,inputTxtChar(i));
        if(isempty(distance))
            distance = distanceVal;
        else
            distance = strcat(distance,':',distanceVal);
        end
        distanceVal = ''; % reset distanceVal
        
    elseif(strcmp(inputTxtChar(i),'k'))
        %disp('Is a k!'); disp(i);
        distanceVal = strcat(distanceVal,inputTxtChar(i));
        
    elseif(strcmp(inputTxtChar(i),'m'))
        %disp('Is an m!'); disp(i);
        if(length(distanceVal)==0) % only m will be in distanceVal
            distanceVal = ''; % reset distanceVal
            
        elseif(i~=len && strcmp(inputTxtChar(i+1),')')) % if next char is ) -> reset distanceVal next iteration
            distanceVal = strcat(distanceVal,inputTxtChar(i));
            %distance = strcat(distance,':',distanceVal);
        
        elseif(i~=len && strcmp(inputTxtChar(i+1),'i')) % if next char is equal to i
            distanceVal = ''; % reset distanceVal, makes sure e.g. 9 min is not added
            
        else
            distanceVal = strcat(distanceVal,inputTxtChar(i));
            %disp('What is distanceVal: '); disp(distanceVal);      
            if(isempty(distance))
                distance = distanceVal;
            else
                distance = strcat(distance,':',distanceVal);
            end
        end
        
    else
        %disp('Nothing important! Reset');
        distanceVal = ''; % not important, reset distanceVal
    end
end


disp('distance: ');
disp(distance);

distanceCellArray = strsplit(distance,':'); % split distance string into separate distance periods

% disp('distanceCellArray: ');
% disp(distanceCellArray);

distArray = zeros(length(distanceCellArray),1); % initialize distance array, int values, [m]
cntr = 1;

for k=1:length(distanceCellArray)
   currDistance = distanceCellArray{k};
   if(strcmp(currDistance(1),'(')) % if first char is '(' -> ignore
       % do nothing
   elseif(strcmp(currDistance(end-1),'k')) % contains [km] -> 
       distArray(cntr) = 1000*str2num(currDistance(1:end-2));
       cntr=cntr+1;
   elseif(strcmp(currDistance(end),'m')) % contains [m]
       distArray(cntr) = str2num(currDistance(1:end-1));
       cntr=cntr+1;
   else
       % do nothing
   end 
end
height = distArray(cntr-2:cntr-1); % last two elements are initial and final height
%height = [0;500];
distArray = distArray(2:cntr-3);
disp('distArray: ');
disp(distArray);


totalDist = sum(distArray);
distCntr = 0;
heightArray = zeros(length(distArray),1);

for i=1:length(distArray)
    distCntr=distCntr+distArray(i);
    if(i==1)
        heightArray(i) = height(1); 
    elseif(i==length(distArray))
        heightArray(i) = height(2); 
    else
       heightArray(i) = interp1([0;totalDist],height,distCntr); % interpolate 
    end
end

%% make a speed profile from the distance profiles

for k=1:length(distArray)
    if(k<length(distArray))
        rise = heightArray(k+1)-heightArray(k);
        run = distArray(k);
        if(rise<=0)
           desAccel = 1; % [m/s^2]
        else % rise>0
            if(run<=rise)
                desAccel = run/rise; % [m/s^2]
            else
                desAccel = rise/run; % [m/s^2]
            end
            desAccel = max(1,desAccel); % makes sure that the desAccel is not too small
        end
    else 
       desAccel = 1; % [m/s^2]
    end
    %disp('desAccel: '); disp(desAccel);
    
    t = 50/(3.6*desAccel);
    if(distArray(k)<(t*50/3.6)) % area of triangle with height 50 km/h
        t = sqrt(distArray(k)/desAccel);
        v = desAccel*t;
        timeArrayLength = ceil(2*t)+1; % rounds up to nearest time
        timeArray = (0:timeArrayLength-1)';
        velArray = zeros(timeArrayLength,1);
        for i=0:timeArrayLength-1
            if(i<=t)
                velArray(i+1) = interp1([0;t],[0;v],i); % (x,v(x),x_interp)
            else % i>t
                velArray(i+1) = interp1([t;ceil(2*t)],[v;0],i); % (x,v(x),x_interp)
            end
        end
    else
        areaRect = distArray(k)-200;
        tRect = areaRect/(50/3.6);
        
        areaTriang = distArray(k)-(50/3.6*tRect);
        v = 50/3.6;
        tTriang = areaTriang/v;
        timeArrayLength = ceil(2*tTriang)+1+ceil(tRect); % rounds up to nearest time
        timeArray = (0:timeArrayLength-1)';
        velArray = zeros(timeArrayLength,1);
        for i=0:timeArrayLength-1
            if(i<=tTriang)
                velArray(i+1) = interp1([0;tTriang],[0;v],i); % (x,v(x),x_interp)
            elseif(i>tTriang && i<=ceil(tTriang+tRect))
                velArray(i+1) = 50/3.6;
            else % i>tTriang+tRect
                velArray(i+1) = interp1([tTriang+tRect+1;timeArrayLength-1],[v;0],i); % (x,v(x),x_interp)
            end
        end
    end
%     disp('Desired distance: '); disp(distArray(k));
%     disp('speed profile distance: '); disp(sum(velArray));
    if(k>2)
        velArray = [velArrayOld;velArray];
    end
    velArrayOld = velArray;
end

timeArray = (1:length(velArray))';
plot(timeArray,3.6*velArray); title('Speed Profile From Google Maps');
xlabel('Time [s]'); ylabel('Speed [km/h]');


%% save timeArray and velArray as text file

velArrayMPH = 2.23694*velArray; % convert from [m/s] -> [m/hr]

A = [timeArray'; velArrayMPH'];

fileID = fopen('/Users/bkvaale/Bjorn/Universities/NTNU/Semester10/RangeEstimation/googleMapsTest.txt','w');
fprintf(fileID,'%6s %12s\n','GoogleMapsTest.txt','Test Profile');
fprintf(fileID,'%6s %12s\n','Test Time [s]','Target Speed [mph]');
fprintf(fileID,'%6.0f\t %12.2f\n',A);
fclose(fileID);



%% misc

% for k=2:length(distArray)
%    currDist = distArray(k);
%    currTime = round(sqrt(currDist/desAccel)); % solve x = x_0+v_0*t + 0.5*a*t^2
%    timeArray = 1:currTime;
%    speedArray = desAccel*timeArray;
%    speedArrayTotal = [speedArray fliplr(speedArray(1:end-1))];
%    disp('sum: '); disp(sum(speedArrayTotal));
%    disp('real value: '); disp(currDist);
% end

% lenCntr=0;
% 
% for k=1:length(distArray)
%     currDist = distArray(k);
%     radius = sqrt(2*currDist/pi);
%     if(radius<=50/3.6)
%         timeArray = 0:(4*50/3.6);
%     else
%         timeArray = 0:2*radius;
%     end
%     velArray = zeros(length(timeArray),1);
%     for i=1:length(timeArray)
%         if(radius<=50/3.6) % max speed should be 50 km/h
%             velArray(i) = sqrt(2*timeArray(i)*radius - timeArray(i)^2); % make speed profile a semi-circle
%         else
%             b = 50/3.6; % make max speed 50 km/h
%             a = 4*b; % choose randomly
%             velArray(i) = b*sqrt(1-(timeArray(i)/a)^2);
%         end
%     end
%     disp('sum: '); disp(sum(velArray));
%     disp('sum int: '); disp(trapz(velArray));
%     disp('real value: '); disp(currDist);
%     if(k>2)
%         velArray = [velArrayOld;velArray];
%     end
%     velArrayOld = velArray;
% 
% %     disp('timeArray: '); disp(timeArray(end));   
% end
% velArray = [velArray;0]; % [m/s]
% velArrayKMH = 3.6*velArray; % [km/h]
% timeArray = (1:length(velArray))';
% 
% figure(1);
% plot(timeArray/60,velArrayKMH); title('Speed Profile From Google Maps');
% xlabel('Time [min]'); ylabel('Speed [km/h]');


