function [actualSpeedMin,actualSpeedMax] = getMinAndMaxSpeedsFromMaxTorqueAndMaxRegenBraking(deltaT,gradeAsDecimal,prevActualSpeed,speedLimit,vehicle)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here

    persistent prevLimMotorSpeedRPM; % remain in memory between function calls, only visible to this function

    if(isempty(prevLimMotorSpeedRPM)) % need to initialize persistent variables
       disp('Is empty!');
       prevLimMotorSpeedRPM = 0; % previous limited motor speed [RPM]
    end

    rho_air = 1.225; % [kg/m^3], air density at 0^o C and 1 atm, see wikipedia, density of air
    %disp('prevActualSpeed: '); disp(prevActualSpeed);
    aeroForce = 0.5*rho_air*vehicle.frontalArea*vehicle.Cd*(prevActualSpeed^2); % rho_air: air density, Cd: drag coefficient

    g = 9.81; % [m/s^2]

    if(abs(prevActualSpeed)>1e-15) % do not use desired speed because it could be the motor cannot achieve that speed, assume very small
        % speeds do not have a rolling force!
       rollingForce = vehicle.wheel.Cr*vehicle.maxVehicleMass*g; % [N]
    else
       rollingForce = 0; % [N] 
    end
    
    gradeAngleInRads = atan(gradeAsDecimal);
    gradeForce = vehicle.maxVehicleMass*g*sin(gradeAngleInRads); % gradeAngleInRads: slope of road, incline: positive, descent: negative
    
    brakeDrag = 0; % assume no frictional losses from brakes
    
    if(abs(prevLimMotorSpeedRPM)>=0 && abs(prevLimMotorSpeedRPM)<=vehicle.drivetrain.motor.ratedRPM) % straight line of torque vs speed curve
        maxMotorTorque = vehicle.drivetrain.motor.maxRatedTorque; % max motor torque is equal to straight line value
    else % rated RPM < demanded motor torque <= max motor RPM, actual motor speed limited in a later equation
        maxMotorTorque = vehicle.drivetrain.motor.maxRatedTorque*(vehicle.drivetrain.motor.ratedRPM/prevLimMotorSpeedRPM);
    end
    
    maxRegenMotorTorque = -min(maxMotorTorque,vehicle.drivetrain.motor.regenFraction*vehicle.drivetrain.motor.maxRatedTorque);
    
    actualAccelForceMin = maxRegenMotorTorque*(vehicle.drivetrain.N/vehicle.wheel.radius) - aeroForce - rollingForce - gradeForce - brakeDrag; % actual acceleration force [N]
    actualAccelForceMax = maxMotorTorque*(vehicle.drivetrain.N/vehicle.wheel.radius) - aeroForce - rollingForce - gradeForce - brakeDrag; % actual acceleration force [N]
 
    actualAccelMin = actualAccelForceMin/vehicle.equivMass; % actual acceleration [m/s^2]
    actualAccelMax = actualAccelForceMax/vehicle.equivMass; % actual acceleration [m/s^2]
    
    testSpeedMin = prevActualSpeed+actualAccelMin*deltaT;
    testSpeedMax = prevActualSpeed+actualAccelMax*deltaT;
  
    motorTestSpeedMin = testSpeedMin*vehicle.drivetrain.N*60/(2*pi*vehicle.wheel.radius);
    motorTestSpeedMax = testSpeedMax*vehicle.drivetrain.N*60/(2*pi*vehicle.wheel.radius);
 
    if(motorTestSpeedMin>=0)
       limMotorSpeedMinRPM = min(motorTestSpeedMin,vehicle.drivetrain.motor.maxRatedRPM);
    else % motorTestSpeedMin < 0
       limMotorSpeedMinRPM = max(motorTestSpeedMin,-vehicle.drivetrain.motor.maxRatedRPM);
    end
    
    if(motorTestSpeedMax>=0)
       limMotorSpeedMaxRPM = min(motorTestSpeedMax,vehicle.drivetrain.motor.maxRatedRPM);
    else % motorTestSpeed < 0
       limMotorSpeedMaxRPM = max(motorTestSpeedMax,-vehicle.drivetrain.motor.maxRatedRPM);
    end
    
    actualSpeedMin = limMotorSpeedMinRPM*2*pi*vehicle.wheel.radius/(60*vehicle.drivetrain.N); % [m/s]
    actualSpeedMax = limMotorSpeedMaxRPM*2*pi*vehicle.wheel.radius/(60*vehicle.drivetrain.N); % [m/s]
    
    if(actualSpeedMin < 0)
        actualSpeedMin = 0; % cannot reverse the car
    end
    
    if(actualSpeedMax > speedLimit)
        actualSpeedMax = speedLimit; % cannot drive faster than speed limit
    end
end

