% code updated from Gregory Plett, Battery Management Systems, Volume II, Equivalent-Circuit Methods," Artech House, 
% 2015.

% EKF updated by Bjorn Kvaale for the Revolve battery cell

function ekfData = initializeEKF(SigmaX0,SigmaV,SigmaW,thetaHatPlus,eta,Q)
  % Initial state description
  ir0   = 0;                           ekfData.irInd = 1;
  hk0   = 0;                           ekfData.hkInd = 2;
  SOC0  = 1;                           ekfData.zkInd = 3; % might want to change this depending on what SOC0 is
  %SOC0  = SoCFromOCV(v0,T0,model); ekfData.zkInd = 3;
  ekfData.xhat  = [ir0 hk0 SOC0]'; % initial state

  % Covariance values
  ekfData.SigmaX = SigmaX0;
  ekfData.SigmaV = SigmaV;
  ekfData.SigmaW = SigmaW;
  ekfData.Qbump = 5;
  
  % previous value of current
  ekfData.priorI = 0;
  ekfData.signIk = 0;
  
  % parameters
  ekfData.Q = Q; % battery capacity at each tested temperature [As]
  ekfData.gamma = thetaHatPlus(2); % hysteresis parameter gamma (unitless)
  ekfData.M = thetaHatPlus(3); % hysteresis parameter [V]
  ekfData.M0 = thetaHatPlus(4); % instantaneous hysteresis parameter [V]
  ekfData.tauRC = thetaHatPlus(1); % RC time constant ???, models a simplified Warburg impedance
  ekfData.R1 = thetaHatPlus(5); % resistance in the RC parallel circuit [Ohm]
  ekfData.R0 = thetaHatPlus(6); % series resistance parameter [Ohm]
  ekfData.eta = eta; % coulombic efficiency
  
