clc; close all; clear

%% import voltage from Excel
filename = '/Users/bkvaale/Bjorn/Universities/NTNU/Semester9/FordypningsProsjekt/Revolve/Batteri Testing/IFE/Celletesting IFE 2017/DynamicTesting/Test8567_dynamic.xlsx';
sheet = 'Test8567';
xlRange = 'AV25:AV16344'; % voltage
voltage = xlsread(filename,sheet,xlRange);
%% import time from Excel
xlRange = 'J25:J16344'; % total time (s)
[~,time,~] = xlsread(filename,sheet,xlRange); % [num,txt,raw] -> ignore num & raw outputs, time: hh:mm:ss:xxx
%% import discharge Sum from Excel
xlRange = 'R25:R16344'; % discharge sum (mAh)
[~,~,dischargeSumRaw] = xlsread(filename,sheet,xlRange); % [num,txt,raw] -> ignore num & txt outputs, dischargeSum: mAh

%% import charge Sum from Excel
xlRange = 'Q25:Q16344'; % charge sum (mAh)
[~,~,chargeSumRaw] = xlsread(filename,sheet,xlRange); % [num,txt,raw] -> ignore num & txt outputs, chargeSum: mAh

%% import current from Excel
xlRange = 'P25:P16344'; % current (A), charge is positive, discharge is negative
[~,~,currentRaw] = xlsread(filename,sheet,xlRange); % [num,txt,raw] -> ignore num & txt outputs, current: A

%% change discharge and charge sums from raw to double
Ltime = length(time);
dischargeSum = zeros(Ltime,1); % in Ah
chargeSum = zeros(Ltime,1); % in Ah
for i = 1:Ltime
    rawValueDisch = dischargeSumRaw{i};
    rawValueChrge = chargeSumRaw{i};
    if(rawValueDisch~=0)
        dischargeSum(i) = str2double(rawValueDisch)/1000;
    else
        dischargeSum(i) = rawValueDisch; % do not divide by 1000, rawValue=0
    end
    if(rawValueChrge~=0)
        chargeSum(i) = str2double(rawValueChrge)/1000;
    else
        chargeSum(i) = rawValueChrge; % do not divide by 1000, rawValue=0
    end
end

%clear rawValueDisch; clear dischargeSumRaw;
%clear rawValueChrge; clear chargeSumRaw;

%% change current from raw to double
current = zeros(Ltime,1);
for i = 1:Ltime
    rawValueCurrent = currentRaw{i};
    if(rawValueCurrent~=0)
        current(i) = str2double(rawValueCurrent);
        if(isnan(current(i))) % if str2double returns NaN
            current(i) = 0; % these elements tend to have very small currents, approximate by 0
        end
    else
        current(i) = rawValueCurrent; % rawValue=0
    end
end

%% format time from hh:mm:ss:xxx -> seconds
sec = zeros(Ltime,1);
for i=1:Ltime
    C = strsplit(time{i},':');
    hours = C{1,1};
    hours = str2double(hours);
    min = C{1,2};
    min = str2double(min);
    ss_xxx = C{1,3};
    %C = strsplit(ss_xxx,'.');
    %ss = C{1,1};
    %ss = str2double(ss);
    %xxx = '.'+C{1,2};
    ss_xxx = str2double(ss_xxx);
    %xxx = str2double(xxx);
    sec(i) = 3600*hours+60*min+round(ss_xxx);
end

%% take away very quick measurements, e.g. if measurements are made in milliseconds
%  do this for time, voltage, dischargeSum, chargeSum & current
secIndex = -1;
secCntr = 0;
secUpdated = zeros(Ltime,1); % updated time (sec)
vUpdated = zeros(Ltime,1); % updated voltage (V)
chargeSumUpdated = zeros(Ltime,1); % updated chargeSum (Ah)
dischargeSumUpdated = zeros(Ltime,1); % updated dischargeSum (Ah)
currentUpdated = zeros(Ltime,1); % updated current (A)
for i=1:Ltime
   if(secIndex~=sec(i))
       secIndex = sec(i);
       secCntr=secCntr+1;
       secUpdated(secCntr)=sec(i);
       vUpdated(secCntr)=voltage(i);
       chargeSumUpdated(secCntr)=chargeSum(i);
       dischargeSumUpdated(secCntr)=dischargeSum(i);
       currentUpdated(secCntr)=current(i);
   end
end

secUpdated = secUpdated(1:secCntr);
vUpdated = vUpdated(1:secCntr);
chargeSumUpdated = chargeSumUpdated(1:secCntr);
dischargeSumUpdated = dischargeSumUpdated(1:secCntr);
currentUpdated = currentUpdated(1:secCntr);
%% update measurements so that cell goes from fully charged->discharged->fully charged
%  do this for time, voltage, dischargeSum, chargeSum & current
%  use find(secUpdated==sec) to find index of sec to start at

% figure(1);
% plot(secUpdated,vUpdated);
% figure(2);
% plot(secUpdated,currentUpdated);

secUpdated = secUpdated(1969:6222); % second != samples -> time different to secUpdated index!
vUpdated = vUpdated(1969:6222);
dischargeSumUpdated = dischargeSumUpdated(1969:6222);
chargeSumUpdated = chargeSumUpdated(1969:6222);
currentUpdated = currentUpdated(1969:6222);
currentUpdated = -1*currentUpdated; % discharge current should be positive, charge current negative

% figure(3);
% plot(secUpdated,vUpdated);
% figure(4);
% plot(secUpdated,currentUpdated);
% figure(5);
% plot(secUpdated,dischargeSumUpdated,secUpdated,chargeSumUpdated);legend('Discharge','Charge');

%% find Q_est

Q_est = (9.0925-2.3008)*3600; % ??? Ah*3600 s/h, found by looking at discharge sum at end of discharge plot

%% find eta @ 25 degrees C
%figure(3);
%plot(sec,dischargeSum,sec,voltage);
% figure(4);
% plot(secUpdated,dischargeSumUpdated,secUpdated,vUpdated); legend('Discharge Sum','Voltage');
% figure(5);
% plot(secUpdated,chargeSumUpdated,secUpdated,vUpdated); legend('Charge Sum','Voltage');

eta_25 = dischargeSumUpdated(end)/chargeSumUpdated(end);

%% make seconds start at 1 instead of 9416
measLength = length(secUpdated);
secUpdated = secUpdated - secUpdated(1)*ones(measLength,1);

%% interpolate seconds, voltage, current

% figure(6);
% plot(secUpdated,vUpdated);
% figure(7);
% plot(secUpdated,currentUpdated);
% figure(8);
% plot(secUpdated,dischargeSumUpdated,secUpdated,chargeSumUpdated);
% legend('Discharge','Charge');

deltaT = 1; % initialize time step vector
t1 = secUpdated(1); t2 = secUpdated(end);
timeEKF = (t1:deltaT:t2)';
vEKF = interp1(secUpdated,vUpdated,(t1:deltaT:t2)'); % interpolated voltage
iEKF = interp1(secUpdated,currentUpdated,(t1:deltaT:t2)'); % interpolated current
dischargeCapEKF = interp1(secUpdated,dischargeSumUpdated,(t1:deltaT:t2)'); % interpolated voltage
chargeCapEKF = interp1(secUpdated,chargeSumUpdated,(t1:deltaT:t2)'); % interpolated current

% figure(9);
% plot(timeEKF,vEKF);
% figure(10);
% plot(timeEKF,iEKF);
% figure(11);
% plot(timeEKF,dischargeCapEKF,timeEKF,chargeCapEKF);
% legend('Discharge','Charge');

%% save data to use in paramEstimationDynamic

filename = 'paramEstDynExcelFormattingOutput.mat';
save(filename,'timeEKF','iEKF','vEKF','eta_25','Q_est','dischargeCapEKF','chargeCapEKF');