function dxdTheta = recursivedxdTheta(k,x,current,thetaHatMinus,deltaT,Q,recLimit)
%UNTITLED6 Summary of this function goes here
%   Detailed explanation goes here
%   Inputs:
%       k: recursive index counter
%       x: vector of all x = [iR1 h z]' values, 3xk
%       current: all measured currents, kx1
%       thetaHatMinus: 6xk
%       deltaT: sampling time, kx1
%       Q: cell capacity
%       kCalled: sample k at which recursivedxdTheta() is first called
%       recLimit: how many samples should recursivedxdTheta() go back
    persistent kCalled; % global only to this function, not entire workspace
    persistent lastdxdTheta; % global only to this function, used for when kCalled-k==recLimit
    %persistent counter; % testing variable

    [rowsX,~] = size(x); % used for determining size of lastdxdThetas
    [rowsTheta,~] = size(thetaHatMinus); % used for determining size of lastdxdThetas
        
    if(isempty(kCalled)==true) % first recursive call, initialize kCalled
%         disp('kCalled empty, initialize kCalled = '); disp(k);
        kCalled = k;
        %counter= 1;
    end
    
    if(isempty(lastdxdTheta)==true) % first recursive call, initialize kCalled
        lastdxdTheta = zeros(rowsX,rowsTheta*recLimit); % keeps the previous recLimit dxdTheta's in storage
        % if k=4 & recLimit=3 -> keeps dxdTheta for k=1, 2 & 3 in
        % storage
    end
     
    if(k==1) % start index k at 1
        dxdTheta = zeros(rowsX,rowsTheta);
        lastdxdTheta = dxdTheta;
        if(kCalled==1)
%             disp('kCalled was cleared for k=='); disp(k);
            clear kCalled;
        end
        return; % stops current function call, returns to recursivedxdtheta(k+1)  
    elseif(kCalled-k==recLimit) % only goes back recLimit samples
        %dxdTheta = zeros(3,6);
        dxdTheta = lastdxdTheta;
%         disp('recLimit called');
        %disp('lastdxdTheta'); disp(lastdxdTheta);
        return; % stops current function call, returns to recursivedxdtheta(k+1)  
    else % k>1
        % find partial f1/partial tau_RC
        tauRCHat = thetaHatMinus(1,k);
        partialARCPartialTauRC = (deltaT/tauRCHat^2)*exp(-deltaT/tauRCHat);
        partialBRCPartialTauRC = -partialARCPartialTauRC;
        prevIR1 = x(1,k-1); % i_R1,k-1
        df1dTauRC = partialARCPartialTauRC*prevIR1+partialBRCPartialTauRC*(current(k-1)); % partial derivative, what about w_k-1 ??
       
        % find partial f1/partial R1
        R1Hat = thetaHatMinus(5,k); C1Hat = tauRCHat/R1Hat;
        partialARCPartialR1 = (deltaT/(R1Hat^2*C1Hat))*exp(-deltaT/(R1Hat*C1Hat));
        partialBRCPartialR1 = -partialARCPartialR1;
        df1dR1 = partialARCPartialR1*prevIR1+partialBRCPartialR1*(current(k-1)); % partial derivative, what about w_k-1 ??
        
        % find partial f2/partial gamma
        prevH = x(2,k-1); % h_k-1
        gammaHat = thetaHatMinus(2,k);
        A_H = exp(-abs((current(k-1)*gammaHat*deltaT)/Q)); % what about w_k-1 ??
        partialAHPartialGamma = -abs((current(k-1)*deltaT)/Q)*A_H; % what about w_k-1 ??
        df2dGamma = partialAHPartialGamma*(prevH+sign(current(k-1))); % what about w_k-1 ??
        
        % find partial f(x_k-1,u_k-1,Theta,w_k-1)/partial x_k-1
        partialfPartialTheta = [df1dTauRC 0 0 0 df1dR1 0;
                                0 df2dGamma 0 0 0 0;
                                0 0 0 0 0 0]; % partial f(x_k-1,u_k-1,Theta,w_k-1)/partial Theta
                            
        % find partial f(x_k-1,u_k-1,Theta,w_k-1)/partial x_k-1
        A_RC = exp(-deltaT/tauRCHat);
        partialfPartialx = [A_RC 0 0;
                            0 A_H 0;
                            0 0 1];
                        
        % find dx/dTheta
        dxdTheta = partialfPartialTheta + partialfPartialx*recursivedxdTheta(k-1,x,current,thetaHatMinus,deltaT,Q,recLimit);
        if(k==kCalled) % most current value of dxdTheta
            lastdxdTheta = dxdTheta;
%             disp('lastdxdThetas updated for k=='); disp(k);
        end
    end
%     disp('kCalled was cleared for k=='); disp(k);
    clear kCalled;
end